--------------------------------------------------------------------------------
--	DIGITALER SINUS-GENERATOR
--
-- Modulname:			dez_to_bin_ram.vhd
-- Projektname:		funktionsgenerator.ise
-- Ersteller:			Thomas Brunnengrber
--	Firma:				Privat
-- Erstellt:			26.03.2006
-- Zielsystem:			Spartan 3
-- Tool versions:		XILINX ISE 8.1
-- Revision:			0.9
-- Abhngigkeiten:	keine
-- 
--------------------------------------------------------------------------------
--	BCD/Binr-Wandler
--	-----------------
-- Beschreibung:
--	Wandler bekommt Adresse des MostSignificantDigit(MSD) bergeben (Start-Adresse).
--	Wandler greift ber den Read-Port auf die Werte im BRAM zu (Dual Port RAM),
-- um der Reihe nach alle Digits einer Variablen auszulesen und gleichzeitig
-- umzurechnen.
-- Das Ende einer Variablen (letztes Digit, LSD) wird durch einen Merker LSD_Flag
-- mitgeteilt. Dieses Digit wird noch bercksichtigt, dann wird der Ausgang
-- "fertig" gesetzt.
-- Die Lnge des Binr-Werts ist fix (generic). Sie muss so gro gewhlt werden,
-- dass der grte bentigte Wert darin Platz hat.
--	Die Wandlung beginnt, wenn Start auf 0 genommen wird. Start darf erst auf 0
-- genommen werden, wenn bei der nchsten positiven Taktflanke ein gltiger
-- BCD-Wert (die hchstwertige Stelle) am BCD-Eingang anliegt.
-- Ist der RAM-Wert an der Start-Adresse ein Minus-Zeichen, so wird der
-- Ausgang "negativ" auf '1' gesetzt, andernfalls auf '0'.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity dez_to_bin_ram is
	generic ( w_adr : integer := 6;
				 l_bin : integer := 32 );
	port ( Takt: in std_logic;
			 Start : in std_logic;
			 StartAdr : in std_logic_vector( w_adr-1 downto 0 );
			 BCD : in std_logic_vector( 3 downto 0 );
			 VZ : in std_logic;	-- Wert ist kein BCD sondern Vorzeichen
										-- VZ = '1' und BCD = '1011' => Plus
										-- VZ = '1' und BCD = '1101' => Minus
			 NaechsteAdr : in std_logic_vector( w_adr-1 downto 0 );
			 LSD_Flag : in std_logic;
			 Adresse : out std_logic_vector( w_adr-1 downto 0 );
			 fertig : out std_logic;
			 negativ : out std_logic;
			 bin_zahl : out std_logic_vector( l_bin-1 downto 0 )
			);
end dez_to_bin_ram ;

architecture behavioral1 of dez_to_bin_ram is
	signal b : unsigned ( l_bin-1 downto 0 );
	signal LSD : std_logic;
begin
	d2b : process ( Takt, Start )
	begin
		if Start = '1' then
			b <= ( others => '0' );
			LSD <= '0';
			negativ <= '0';
		elsif rising_edge( Takt ) then
			if (VZ = '1') then
				if BCD(2) = '1' then
					negativ <= '1';
				end if;
			elsif (LSD = '0') then
				b <= ( ( b( l_bin-4 downto 0 ) & "00" ) 	-- b := (b*4+b)*2 + nchstes_digit
					  + ( b( l_bin-2 downto 0 ) ) & "0" )	--   := b*10 + nchstes_digit
					  + unsigned ( BCD );
			end if;
			if LSD_Flag = '1' then
				LSD <= '1';
			end if;
		end if;
	end process d2b;
	bin_zahl <= std_logic_vector( b );
	
	fertig <= LSD;
	
	Adresse <= StartAdr when Start = '1'
					else NaechsteAdr;
					
end behavioral1;
