--------------------------------------------------------------------------------
--	DIGITALER SINUS-GENERATOR
--
-- Modulname:			func_gen_main.vhd
-- Projektname:		funktionsgenerator.ise
-- Ersteller:			Thomas Brunnengrber
--	Firma:				Privat
-- Erstellt:			27.03.2006
-- Zielsystem:			Spartan 3
-- Tool versions:		XILINX ISE 8.1
-- Revision:			1.0
-- Abhngigkeiten:	sinus_generator.vhd
--							user_interface.vhd
--							tda1543_interface.vhd
--							func_gen_main.ucf
-- 						taktteiler2c.vhd (nur fr Test)
--							bcd_to_7seg.vhd (nur fr Test)
--------------------------------------------------------------------------------
--	Hauptdatei des Funktionsgenerators
--
-- Allgemeine Daten:
--
-- Zwei unabhngige Kanle
--
-- Frequenzauflsung: 0,01 Hz
-- Begrenzung auf 99,99999 kHz wg. D/A-Wandler, mit anderem D/A-Wandler
-- knnte ca. 100MHz Sampling-Frequenz erreicht werden.
--		=> d_phase fr 99,99999 kHz = 9999999 => 24 bit
--
-- Amplitude max.: 0,000 V bis 2,047 V	(begrenzt durch analoge Ausgangstufe)
-- Amplitudenauflsung: 0,001 V
-- 1 LSB = 1 mV => 11 bit breit
--
-- Offset max.: -2,047 bis +2,047 V		(begrenzt durch analoge Ausgangstufe)
-- Offsetauflsung: 0,001 V
-- 1 LSB = 1 mV => 11 bit breit zuzglich Vorzeichen
--
-- Weitere Dokumentation in den Dateien
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

library UNISIM;
use UNISIM.VComponents.ALL;

entity func_gen_main is
port (
	Quarz 		: in std_logic;		-- 50MHz
	Hoch, Runter, Rechts, Links, Eingabe : in std_logic;
	-- LCD:
	LCD_Enable1	: out std_logic := '0';					-- LCD Enable
	LCD_Enable2	: out std_logic := '0';					-- LCD Enable
	LCD_RS		: out std_logic;							-- LCD register select
	LCD_R_W		: out std_logic;							-- LCD Read / nWrite
	LCD_Daten	: out std_logic_vector(7 downto 0);	-- Datenbyte zum LCD
	Vo_Takt		: out std_logic;							-- Takt fr Spannungswandler	
	-- D/A-Wandler:
	DAC_Takt		: out std_logic;							-- Takt fr serielle Daten
	DAC_Data 	: out std_logic;							-- serielle Daten
	DAC_WS 		: out std_logic;							-- Kanal
	-- Sonstiges:
	SRAM1_CE		: out std_logic := '1';					-- damit SRAM hochohmig
	SRAM2_CE		: out std_logic := '1';					-- damit SRAM hochohmig
test_anoden		: out std_logic_vector(3 downto 0);
test_segments	: out std_logic_vector(0 to 6);
test_dp			: out std_logic
--	Anoden		: out std_logic_vector ( 1 to 4 ) := "1111"	-- damit LEDs aus!
);

	constant n_Tasten : integer := 5;
	-- Taktfrequenz 2^32/100 Hz = 42949672.96 Hz ergibt Auflsung 0.01 Hz und w_phase=32
	-- Alternativ z. B. Takt 2^29/10 Hz = 53687091.2 Hz ergibt Auflsung 0.1 Hz
	--	und bentigte w_phase=29 oder z. B. Takt 2^25 Hz = 33554432 Hz ergibt
	-- Auflsung 1 Hz und bentigte w_phase=25.
	constant w_phase : integer := 32;	-- Festgelegt durch Taktfrequenz und Auflsung!!!
	constant w_d_phase : integer := 24;	-- Breite der Phasendifferenz zwischen zwei Takten
													-- => vom Benutzer gewhlte Frequenz
	constant w_ampl : integer := 11;		-- Breite der vom Benutzer gewhlten Amplitude
	-- Maximum der Breiten der vom Benutzer einstellbaren Werte d_phase, ampl, offs:
	constant w_max : integer := 24;	
	constant w_dac : integer := 16;		-- Auflsung des D/A-Wandlers

end entity func_gen_main;

architecture arch1 of func_gen_main is
									
	component sinus_generator is
		generic (
			w_phase : integer := 12;
			w_d_phase : integer := 10;
			w_fktwert : integer := 6 );
		port (
			takt : in std_logic;
			d_phase : in std_logic_vector ( w_d_phase-1 downto 0 );
			neustart : in std_logic;	-- Trigger bei	steigender Flanke
			start_phase : in std_logic_vector ( w_phase-1 downto 0 ); 
					-- bernahme synchron mit takt, wenn neustart = '1'
			fkt_wert : out std_logic_vector ( w_fktwert-1 downto 0 ) -- Tabelleneintrag
		);
	end component sinus_generator;

	component tda1543_interface is
		generic ( w_daten : integer := 16;
					 w_amplitude : integer := 11 );
		port ( takt_gen : in std_logic;	-- Takt des Funktionsgenerators (ca. 43 MHz)
													-- wird fr die Synchronisation bentigt
				 takt_dac : in std_logic;	-- Takt fr die Ausgabe an den DAC (ca. 9 MHz)
				 reset : in std_logic;
				 kanal1 : in std_logic_vector( w_daten-1 downto 0 );
				 kanal2 : in std_logic_vector( w_daten-1 downto 0 );
				 a1, a2 : in std_logic_vector( w_amplitude-1 downto 0 );
				 o1, o2 : in std_logic_vector( w_amplitude downto 0 );
				 ws : out std_logic;
	test_da_daten_k1 : out std_logic_vector( w_daten-1 downto 0 );
				 data : out std_logic );
	end component tda1543_interface;
	
	component user_interface is
	generic (
		n_Tasten : integer := 5;
		-- Breiten der Ausgnge:
		w_d_phase : integer := 30;		-- Breite der Phasendifferenz zwischen zwei Takten
		w_ampl : integer := 11;			-- Breite der Amplitude
		w_max : integer := 30			-- Maximum aus allen Breiten
	);
	port (
		takt : in std_logic;
		reset : in std_logic;
		tasten : in std_logic_vector( 1 to n_Tasten );	-- Tasten (nicht entprellt)
		-- Signale fr Display:
		Osz_Vo		: out	std_logic;							-- Oszillator fr Spannungswandler
		LCD_Daten	: out std_logic_vector(7 downto 0); -- Datenbyte zum LCD
		LCD_RS		: out std_logic := '0';					-- LCD register select
		LCD_R_W		: out std_logic := '0';					-- LCD Read / nWrite
		LCD_Enable1	: out std_logic := '0';					-- LCD Enable
		LCD_Enable2	: out std_logic := '0';					-- LCD Enable
		-- Signale fr Funktionsgenerator:
	--	neustart : out std_logic; -- Nur fr Generator mit einer Frequenz und fester
			-- Phasenverschiebung nach nderung der Phase oder Frequenz bentigt.
		d_phase1, d_phase2 : out std_logic_vector ( w_d_phase-1 downto 0 );
		ampl1, ampl2 : out std_logic_vector ( w_ampl-1 downto 0 ); 
		offs1, offs2 : out std_logic_vector ( w_ampl downto 0 )
	);
	end component user_interface;

	component taktteiler2c is
		generic ( n : integer range 1 to 30 := 2 );	-- Takt-Teilung durch 2^n 
		port ( loeschen : in std_logic;
				 Takt_ein : in std_logic;
				 Takt_aus : out std_logic );
	end component taktteiler2c;

	component bcd_to_7seg is
    port ( enable : in std_logic;
	 		  bcd : in std_logic_vector(3 downto 0);
           led : out std_logic_vector(0 to 6) );
	end component bcd_to_7seg;

	constant CLKDV_DIVIDE1 : real := 5.5; 	-- 50 / 5,5 = 9,09 MHz
	constant CLKFX_MULTIPLY1 : integer := 17;
	constant CLKFX_MULTIPLY2 : integer := 24;
	constant CLKFX_DIVIDE1 : integer := 19;
	constant CLKFX_DIVIDE2 : integer := 25;

	signal dcm2_locked : std_logic;
	signal takt_dac : std_logic;	-- Takt fr DA-Wandler, von DCM1 generiert.
	signal takt_1 : std_logic;		-- Ausgang von DCM_1	zu DCM_2
	signal osc_dcm1 : std_logic;	-- nur fr Feedback, unntig
	signal takt_gen : std_logic;	-- Takt fr Generator, ideal: 42949672.96 Hz
		-- Ausgnge der Funktionsgeneratoren fr den DAC:
	signal f_wert1, f_wert2 : std_logic_vector( w_dac-1 downto 0 );
		-- Ausgnge des Benutzer-Interface fr die Funktionsgeneratoren:
	signal d_phase1, d_phase2 : std_logic_vector( w_d_phase-1 downto 0 );
	constant start_phase : std_logic_vector( w_phase-1 downto 0 ) := (others => '0');
		-- Ausgnge des Benutzer-Interface fr den DAC:
	signal ampl1, ampl2 : std_logic_vector( w_ampl-1 downto 0 );
	signal offs1, offs2 : std_logic_vector( w_ampl downto 0 );
		
		-- Testsignale:
	signal test_da_daten : std_logic_vector ( w_dac-1 downto 0 );
	signal sseg_wert : std_logic_vector ( 3 downto 0 );
	signal sseg_takt : std_logic;		-- clock for display (1ms)
	signal sseg_stelle : natural range 0 to 3 := 0 ;
		
begin

   -- DCM: Digital Clock Manager Circuit for Virtex-II/II-Pro and Spartan-3/3E
   -- Xilinx HDL Language Template version 7.1i
   DCM_1 : DCM
   generic map (
      CLKDV_DIVIDE => CLKDV_DIVIDE1, 
		CLKFX_DIVIDE => CLKFX_DIVIDE1,
      CLKFX_MULTIPLY => CLKFX_MULTIPLY1,
      CLKIN_DIVIDE_BY_2 => FALSE, CLKIN_PERIOD => 20.0, 
      CLKOUT_PHASE_SHIFT => "NONE", CLK_FEEDBACK => "1X",
      DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
      DFS_FREQUENCY_MODE => "LOW", DLL_FREQUENCY_MODE => "LOW",
      DUTY_CYCLE_CORRECTION => TRUE, FACTORY_JF => X"C080",
      PHASE_SHIFT => 0, STARTUP_WAIT => FALSE)
   port map (
      CLK0 => osc_dcm1, CLK180 => open, CLK270 => open,
      CLK2X => open, CLK2X180 => open, CLK90 => open,
      CLKDV => takt_dac,
		CLKFX => takt_1,   -- DCM CLK synthesis out (M/D)
      CLKFX180 => open, LOCKED => open, PSDONE => open,
		STATUS => open, CLKFB => osc_dcm1,
		CLKIN => quarz,   -- Clock input (from IBUFG, BUFG or DCM)
      PSCLK => '0', PSEN => '0', PSINCDEC => '0', RST => '0'
   );	  	-- End of DCM_1 instantiation
   DCM_2 : DCM
   generic map (
      CLKDV_DIVIDE => 2.0, 
		CLKFX_DIVIDE => CLKFX_DIVIDE2,
      CLKFX_MULTIPLY => CLKFX_MULTIPLY2,
      CLKIN_DIVIDE_BY_2 => FALSE, CLKIN_PERIOD => 22.35, 
      CLKOUT_PHASE_SHIFT => "NONE", CLK_FEEDBACK => "NONE",
      DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
      DFS_FREQUENCY_MODE => "LOW", DLL_FREQUENCY_MODE => "LOW",
      DUTY_CYCLE_CORRECTION => TRUE, FACTORY_JF => X"C080",
      PHASE_SHIFT => 0, STARTUP_WAIT => FALSE)
   port map (
      CLK0 => open, CLK180 => open, CLK270 => open,
      CLK2X => open, CLK2X180 => open, CLK90 => open,
      CLKDV => open,
		CLKFX => takt_gen,   -- DCM CLK synthesis out (M/D)
      CLKFX180 => open, LOCKED => dcm2_locked, PSDONE => open,
		STATUS => open, CLKFB => open,
		CLKIN => takt_1,   -- Clock input (from IBUFG, BUFG or DCM)
      PSCLK => '0', PSEN => '0', PSINCDEC => '0', RST => '0'
   );	  	-- End of DCM_2 instantiation

	fgen1 : sinus_generator
	generic map ( w_phase => w_phase, w_d_phase => w_d_phase, w_fktwert => w_dac )
	port map ( takt => takt_gen, d_phase => d_phase1, neustart => not dcm2_locked,
				start_phase => start_phase, fkt_wert => f_wert1 );

	fgen2 : sinus_generator
	generic map ( w_phase => w_phase, w_d_phase => w_d_phase, w_fktwert => w_dac )
	port map ( takt => takt_gen, d_phase => d_phase2, neustart => not dcm2_locked,
				start_phase => start_phase, fkt_wert => f_wert2 );

	da1_interface : tda1543_interface
		generic map ( w_daten => w_dac, w_amplitude => w_ampl )
		port map ( takt_gen => takt_gen, takt_dac => takt_dac,
				 reset => not dcm2_locked, kanal1 => f_wert1, kanal2 => f_wert2,
				 a1 => ampl1, a2 => ampl2, o1 => offs1, o2 => offs2,
				 ws => DAC_ws, test_da_daten_k1 => test_da_daten, data => DAC_data );
	
	ui : user_interface
	generic map ( n_Tasten => n_Tasten, w_d_phase =>w_d_phase,
		w_ampl => w_ampl, w_max => w_max )
	port map ( takt => takt_gen, reset => not dcm2_locked,
		tasten(1) => Eingabe, tasten(2) => Hoch, tasten(3) => Runter,
		tasten(4) => Rechts, tasten(5) => Links,
		Osz_Vo => Vo_Takt, LCD_Daten => LCD_Daten,
		LCD_RS => LCD_RS, LCD_R_W => LCD_R_W,
		LCD_Enable1	=> LCD_Enable1, LCD_Enable2 => LCD_Enable2,
		d_phase1 => d_phase1, d_phase2 => d_phase2,
		ampl1 => ampl1, ampl2 => ampl2, offs1 => offs1, offs2 => offs2	);

	DAC_Takt <= not takt_dac;
	SRAM1_CE	<= '1';		-- damit SRAM hochohmig
	SRAM2_CE	<= '1';		-- damit SRAM hochohmig
--	Anoden <= "1111";		-- damit LEDs aus!

-- Testanzeige:
	sieben_seg_takt : taktteiler2c
		generic map ( n => 15 )		-- 43MHz / 2^15 = ca. 1,3 kHz
		port map ( loeschen => '0', Takt_ein => takt_dac, Takt_aus => sseg_takt );

	digit_coder : bcd_to_7seg
		port map ( enable => '1', bcd => sseg_wert, led => test_segments );

	sieben_seg_anzeige : process ( sseg_takt )
		variable anoden_intern : unsigned(3 downto 0) := ( 0 => '0', others => '1');
	begin
		if rising_edge( sseg_takt ) then
			if sseg_stelle = 3 then
				sseg_stelle <= 0;
			else
				sseg_stelle <= sseg_stelle + 1;
			end if;
			anoden_intern := anoden_intern rol 1;
		end if;
		test_anoden <= std_logic_vector( anoden_intern );
	end process;
	sseg_wert <= test_da_daten( 4*sseg_stelle+3 downto 4*sseg_stelle );
	test_dp <= '1';
	
end arch1;
