--------------------------------------------------------------------------------
--	DIGITALER SINUS-GENERATOR
--
-- Modulname:			lcd_interface.vhd
-- Projektname:		funktionsgenerator.ise
-- Ersteller:			Thomas Brunnengrber
--	Firma:				Privat
-- Erstellt:			19.03.2006
-- Zielsystem:			Spartan 3
-- Tool versions:		XILINX ISE 8.1
-- Revision:			1.0
-- Abhngigkeiten:	taktteiler2c.vhd
--							anzeigen_timer.vhd
-- 
--------------------------------------------------------------------------------
--	Interface fr LC-Display mit 2 Controllern/4 Zeilen
--	---------------------------------------------------
-- 
--	Reset erfolgt im Controller (Spannungsversorgung muss stimmen).
--	Busy-Bit wird nicht gelesen.
-- Eingang reset: 
--		Bei neg. Flanke wird das Display neu initialisiert.
-- Eingang befehl: 
--		0: Position setzen
-- 	1: Zeichen schreiben (Position wird zunchst nach dem Schreiben
--			vom Controller erhht, anschlieend aber wieder zurckgesetzt)
--		4: Cursor aus, Blinken aus
--		5: Cursor aus, Blinken ein
-- 	6: Cursor ein, Blinken aus
--		2,3,7: unbenutzt
-- Eingang position:
--		Bit 7: Auswahl Controller ('0' => Controller 1 fr Zeilen 1/2,
--				 '1' => Controller 2 fr Zeilen 3/4)
--		Bit 6..0: Position, 00h: 1. Zeichen 1./3. Zeile, 40h: 1. Zeichen 2./4. Zeile
--	Eingang starten:
--		Befehlsausfhrung startet, wenn dieser Eingang gesetzt ist und vorherige
--		Befehle abgearbeitet wurden.
--		Eingang darf erst zurckgenommen werden, wenn Ausgang busy auf 1 gegangen
--		ist.
--------------------------------------------------------------------------------

library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;

entity LCD_Interface is
	port( Takt			: in std_logic;	-- 20-50 MHz
			reset			: in std_logic;	-- Asynchroner Reset
			befehl		: in std_logic_vector( 2 downto 0 );
			position		: in std_logic_vector( 7 downto 0 ); 
					-- Fr befehl=0 und befehl=1 vollstndig bentigt,
					-- fr befehl=2, 3 und 4 nur position(7).
			zeichen		: in std_logic_vector( 7 downto 0 ); -- Nur fr befehl = 1	
			starten		: in std_logic;			-- Befehlsausfhrung starten
			busy			: out std_logic;			-- '1', wenn LCD_Interface beschftigt
			Osz_Vo		: out	std_logic;			-- Oszillator fr Spannungswandler
			LCD_Daten	: out std_logic_vector( 7 downto 0 ); -- Datenbyte zum LCD
			LCD_RS		: out std_logic := '0';	-- LCD register select
			LCD_R_W		: out std_logic := '0';	-- LCD Read / nWrite
			LCD_Enable1	: out std_logic := '0';	-- LCD Enable Zeile 1/2
			LCD_Enable2	: out std_logic := '0'	-- LCD Enable Zeile 3/4
		 );
end LCD_Interface;

-- Intern bentigte Befehle:
--	1. Initialisierung:
--		nach LCD-Start >40ms warten => FPGA braucht lnger => erfllt
--		Funktion 8bit Port, 2 Zeilen, 5x7 => RS = 0, b"0011_1000"
--	2. Position setzen: RS = 0, b"1xxx_xxxx"	(xxx_xxxx = Adresse)
--	3. Zeichen setzen: RS = 1, ASCII-Code	
--	4.	Display ein, Cursor aus, Blinken aus => RS = 0, b"0000_1100"
--	5. Display ein, Cursor aus, Blinken ein => RS = 0, b"0000_1101"	
--	6. Display ein, Cursor ein, Blinken aus => RS = 0, b"0000_1110"	

architecture arch1 of LCD_Interface is

	component Anzeigen_Timer is
		port( Takt			: in std_logic;				-- 1-2 MHz
				reset			: in std_logic;				-- Asynchroner Reset, active high
				start			: in std_logic;				-- Starte Schreib-Sequenz
				busy			: out std_logic := '0';		-- '1' solange LCD beschftigt
				LCD_Enable	: out std_logic := '0' );	-- LCD Enable
	end component Anzeigen_Timer;

	component taktteiler2c is
		generic ( n : integer range 1 to 30 := 2 );	-- Takt-Teilung durch 2^n 
		port ( loeschen : in std_logic;
				 Takt_ein : in std_logic;
				 Takt_aus : out std_logic );
	end component taktteiler2c;

	type Zustandstyp is ( Reset_Ein, 
				Setze_Funktion1, Setze_Funktion2, -- Setze_Funktion3, Setze_Funktion4,
								 Leerlauf,
								 Setze_Zeichen, Setze_Position, 
								 CAus_BAus, CEin_BAus, CAus_BEin );
	signal Zustand : Zustandstyp;
	signal Timer_Takt	: std_logic;			-- max. 50/32 MHz bzw. 20ns*32=640ns
	signal Starte_LCD_Schreiben : std_logic;
	signal LCD_busy : std_logic;
	signal LCD_Enable : std_logic;
	signal busy_int : std_logic;

begin
	LCD_R_W <= '0';		-- es wird nur geschrieben

	Timer_Taktteiler : taktteiler2c
		-- Timer bentigt Teilung durch 32 = 2^5
		generic map ( n => 5 )	 -- 5
	   port map ( loeschen => reset, Takt_ein => Takt, 
					  Takt_aus => Timer_Takt );

	Osz_Teiler : taktteiler2c
		-- Oszillator bentigt ca. 50kHz => Teilung durch 1024 = 2^10
		generic map ( n => 10 )	 -- 10
	   port map ( loeschen => reset, Takt_ein => Takt, 
					  Takt_aus => Osz_Vo );

	AT1 : Anzeigen_Timer
		port map ( Takt => Timer_Takt, reset => reset, start => Starte_LCD_Schreiben,
					  busy => LCD_busy, LCD_Enable => LCD_Enable );

	fsm1 : process ( Takt, reset )
	begin
		if reset = '1' then
			busy_int <= '1';
			Zustand <= Reset_Ein;
		elsif rising_edge( Takt ) then
			if LCD_busy = '0' then	-- kein Schreibzyklus im Gang
				if busy_int = '1' then	-- Schreibzyklus gerade abgeschlossen
					case Zustand is
						when Reset_Ein =>
							Zustand <= Setze_Funktion1;
							Starte_LCD_Schreiben <= '1';	-- neuen Schreibzyklus beginnen
						when Setze_Funktion1 =>
							Zustand <= Setze_Funktion2;
							Starte_LCD_Schreiben <= '1';	-- neuen Schreibzyklus beginnen
--						when Setze_Funktion2 =>
--							Zustand <= Setze_Funktion3;
--							Starte_LCD_Schreiben <= '1';	-- neuen Schreibzyklus beginnen
--						when Setze_Funktion3 =>
--							Zustand <= Setze_Funktion4;
--							Starte_LCD_Schreiben <= '1';	-- neuen Schreibzyklus beginnen
						when Setze_Zeichen =>
							Zustand <= Setze_Position;
							Starte_LCD_Schreiben <= '1';	-- neuen Schreibzyklus beginnen
						when others =>
							Zustand <= Leerlauf;
							busy_int <= '0';
					end case;
				elsif Zustand = Leerlauf and starten = '1' then
					case befehl is
						when "000" =>
							Zustand <= Setze_Position;
						when "001" =>
							Zustand <= Setze_Zeichen;
						when "100" =>
							Zustand <= CAus_BAus;
						when "101" =>
							Zustand <= CAus_BEin;
						when "110" =>
							Zustand <= CEin_BAus;
						when others => 	-- Fehler
							Zustand <= Leerlauf;
					end case;
					busy_int <= '1';
					Starte_LCD_Schreiben <= '1';
				end if;				
			else		-- Schreibzyklus in Gang
				Starte_LCD_Schreiben <= '0';
			end if;
		end if;
	end process;
	
	LCD_Enable1 <= LCD_Enable when ( Zustand = Setze_Funktion1 
								or Zustand = Setze_Funktion2 
--								or Zustand = Setze_Funktion3
--								or Zustand = Setze_Funktion4
								or Zustand = CAus_BAus 
								or Position(7) = '0') 
						else '0';
	LCD_Enable2 <= LCD_Enable when ( Zustand = Setze_Funktion1 
								or Zustand = Setze_Funktion2
--								or Zustand = Setze_Funktion3
--								or Zustand = Setze_Funktion4
								or Zustand = CAus_BAus 
								or Position(7) = '1')
						else '0';
	LCD_RS <= '1' when Zustand = Setze_Zeichen 
--								or Zustand = Setze_Funktion4
						else '0';					 
	
	with Zustand select
	LCD_Daten <= "00111000" when Setze_Funktion1,
					 "00111000" when Setze_Funktion2,
--					 "10000000" when Setze_Funktion3,	-- Position 0
--					 zeichen when Setze_Funktion4,		-- Zeichen "zeichen"
					 ('1' & position(6 downto 0)) when Setze_Position,
					 zeichen when Setze_Zeichen,
					 "00001100" when CAus_BAus,
					 "00001101" when CAus_BEin,
					 "00001110" when CEin_BAus,
					 "--------" when others;			-- egal
	busy <= busy_int;

end arch1;

--architecture arch2 of LCD_Interface is
--
--	component taktteiler2c is
--		generic ( n : integer range 1 to 30 := 2 );	-- Takt-Teilung durch 2^n 
--		port ( loeschen : in std_logic;
--				 Takt_ein : in std_logic;
--				 Takt_aus : out std_logic );
--	end component taktteiler2c;
--
--	component Anzeigen_Timer is
--		port( Takt			: in std_logic;							-- 1-2 MHz
--				reset			: in std_logic;							-- Asynchroner Reset, active high
--				start			: in std_logic;							-- Starte Schreib-Sequenz
--				busy			: out std_logic := '0';					-- '1' solange LCD beschftigt
--				LCD_Enable	: out std_logic := '0'					-- LCD Enable
--			 );
--	end component Anzeigen_Timer;
--
--	signal Timer_Takt	: std_logic;
--	signal Starte_LCD_Schreiben : std_logic;
--	signal LCD_busy : std_logic;
--
--begin
--
--	LCD_R_W <= '0';		-- es wird nur geschrieben
--	LCD_Enable2 <= '0';
--
--	Timer_Taktteiler : taktteiler2c
--		-- Timer bentigt Teilung durch 32 = 2^5
--		generic map ( n => 5 )	 
--	   port map ( loeschen => reset, Takt_ein => Takt, 
--					  Takt_aus => Timer_Takt );
--
--	Osz_Teiler : taktteiler2c
--		-- Oszillator bentigt ca. 50kHz => Teilung durch 1024 = 2^10
--		generic map ( n => 10 )	 -- 10
--	   port map ( loeschen => reset, Takt_ein => Takt, 
--					  Takt_aus => Osz_Vo );
--
--	AT : Anzeigen_Timer
--		port map ( Takt => Timer_Takt, reset => reset, start => Starte_LCD_Schreiben,
--					  busy => LCD_busy, LCD_Enable => LCD_Enable1 );
--
--	LCD_schreiben : process ( Takt, reset )
--		type Zustands_Typ is ( Start, Setze_Funktion, Display_ein,
--									  Schreibe_Zeichen, Setze_Position, 
--									  Blinken_ein, Blinken_aus, Leerlauf );
--		variable Zustand : Zustands_Typ;
--	begin
--		if reset = '1' then
--			busy <= '1';
--			Zustand := Start;
----			test_zustand <= "001";					
--		elsif rising_edge( Takt ) then
--			case Zustand is
--				when Start =>
--						Zustand := Setze_Funktion;
----						test_zustand <= "010";					
--						LCD_Daten <= b"0011_1000";	-- Funktion setzen
--						LCD_RS <= '0';					-- Steuer-Register
--						Starte_LCD_Schreiben <= '1';
--				when Setze_Funktion =>
--					if LCD_busy = '1' then
--						Starte_LCD_Schreiben <= '0';
--					else
--						Zustand := Display_ein;
----						test_zustand <= "011";					
--						LCD_Daten <= b"0000_1110";	-- Display ein, Cursor ein, Blinken aus
--						LCD_RS <= '0';					-- Steuer-Register
--						Starte_LCD_Schreiben <= '1';						
--					end if;
--				when Display_ein =>
--					if LCD_busy = '1' then
--						Starte_LCD_Schreiben <= '0';
--					else
--						Zustand := Schreibe_Zeichen;
----						test_zustand <= "100";					
--						LCD_Daten <= b"0100_0001";	-- 'A'
--						LCD_RS <= '1';					-- Zeichen-Register
--						Starte_LCD_Schreiben <= '1';						
--					end if;
--				when Schreibe_Zeichen =>
--					if LCD_busy = '1' then
--						Starte_LCD_Schreiben <= '0';
--					else
--						Zustand := Setze_Position;
----						test_zustand <= "101";					
--						LCD_Daten <= b"1100_0000";	-- Cursor auf Zeile 2, Zeichen 1
--						LCD_RS <= '0';					-- Kontroll-Register
--						Starte_LCD_Schreiben <= '1';						
--					end if;
--				when Setze_Position =>
--					if LCD_busy = '1' then
--						Starte_LCD_Schreiben <= '0';
--					else
--						Zustand := Blinken_ein;
----						test_zustand <= "110";					
--						LCD_Daten <= b"0000_1101";	-- Display ein, Cursor aus, Blinken ein
--						LCD_RS <= '0';					-- Kontroll-Register
--						Starte_LCD_Schreiben <= '1';						
--					end if;
--				when Blinken_ein =>
--					if LCD_busy = '1' then
--						Starte_LCD_Schreiben <= '0';
--					else
--						busy <= '0';
--						Zustand := Leerlauf;
----						test_zustand <= "000";					
--					end if;
--				when Blinken_aus => null;
--				when Leerlauf =>
--						busy <= '0';
--						Zustand := Leerlauf;
----						test_zustand <= "000";					
--			end case;
--		end if;
----		test_LCD_busy <= LCD_busy;
----		test_Starte_LCD_Schreiben <= Starte_LCD_Schreiben;
----		test_Timer_Takt <= Timer_Takt;		
--	end process;
--end arch2;
